@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2024 Intel Corporation
+Copyright (C) 2020-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -101,7 +101,11 @@ static SPIRVArgDesc parseImageType(StringRef TyName) {
   std::tie(ImageType, TyName) = parseImageDim(TyName);
   AccessType AccType;
   std::tie(AccType, TyName) = parseAccessQualifier(TyName);
-  assert(TyName == CommonTypes::TypeSuffix && "Bad image type");
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+  assert(TyName.starts_with(CommonTypes::TypeSuffix) && "Bad image type");
+#else
+  assert(TyName.startswith(CommonTypes::TypeSuffix) && "Bad image type");
+#endif
   return {ImageType, AccType};
 }
 
@@ -141,7 +145,7 @@ static SPIRVType evaluateImageTypeFromSPVIR(SPIRVIRTypes::Dim Dim,
       break;
     case SPIRVIRTypes::DimBuffer:
       ResultType = SPIRVType::Image1dBuffer;
-      break; 
+      break;
     }
   } else {
     switch (Dim) {
@@ -176,6 +180,9 @@ static SPIRVArgDesc parseSPIRVIRImageType(StringRef TyName) {
   // SPIRV friendly Ir image type looks like this:
   // spirv.Image._{Sampled T}_{Dim}_{Depth}_{Arrayed}_{MS}_{Fmt}_{Acc}
 
+  // skip dot
+  TyName = TyName.drop_front(1);
+
   // skip Samled Type.
   TyName = skipUnderscores(TyName, 2);
 
@@ -207,7 +214,11 @@ static VCINTR::Optional<SPIRVArgDesc> parseIntelType(StringRef TyName) {
   std::tie(MainType, TyName) = parseIntelMainType(TyName);
   AccessType AccType;
   std::tie(AccType, TyName) = parseAccessQualifier(TyName);
-  assert(TyName == CommonTypes::TypeSuffix && "Bad intel type");
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+  assert(TyName.starts_with(CommonTypes::TypeSuffix) && "Bad intel type");
+#else
+  assert(TyName.startswith(CommonTypes::TypeSuffix) && "Bad intel type");
+#endif
   return SPIRVArgDesc{MainType, AccType};
 }
 
@@ -217,7 +228,11 @@ static VCINTR::Optional<SPIRVArgDesc> parseOCLType(StringRef TyName) {
 
   // Sampler type.
   if (TyName.consume_front(OCLTypes::Sampler)) {
-    assert(TyName == CommonTypes::TypeSuffix && "Bad sampler type");
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+    assert(TyName.starts_with(CommonTypes::TypeSuffix) && "Bad sampler type");
+#else
+    assert(TyName.startswith(CommonTypes::TypeSuffix) && "Bad sampler type");
+#endif
     return {SPIRVType::Sampler};
   }
 
@@ -256,6 +271,37 @@ static VCINTR::Optional<SPIRVArgDesc> parseOpaqueType(StringRef TyName) {
   return parseSPIRVIRType(TyName);
 }
 
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+static SPIRVArgDesc analyzeTargetExtTypeArg(const Argument &Arg,
+                                            TargetExtType *TET) {
+  auto TyName = TET->getName();
+  if (TyName.consume_front(SPIRVIRTypes::TypePrefix)) {
+    if (TyName.consume_front(SPIRVIRTypes::Sampler))
+      return {SPIRVType::Sampler};
+    if (TyName.consume_front(SPIRVIRTypes::Buffer)) {
+      assert(TET->getNumIntParameters() == 1);
+      auto Acc = static_cast<AccessType>(TET->getIntParameter(0));
+      return SPIRVArgDesc(SPIRVType::Buffer, Acc);
+    }
+    if (TyName.consume_front(SPIRVIRTypes::Image)) {
+      auto Dim = static_cast<SPIRVIRTypes::Dim>(
+          TET->getIntParameter(SPIRVIRTypes::Dimension));
+      auto Arr = static_cast<bool>(TET->getIntParameter(SPIRVIRTypes::Arrayed));
+      auto Acc =
+          static_cast<AccessType>(TET->getIntParameter(SPIRVIRTypes::Access));
+      auto SpvTy = evaluateImageTypeFromSPVIR(Dim, Arr);
+      if (SpvTy == SPIRVType::Image2d &&
+          Arg.getParent()->getAttributes().hasParamAttr(
+              Arg.getArgNo(), VCFunctionMD::VCMediaBlockIO))
+        SpvTy = SPIRVType::Image2dMediaBlock;
+      return SPIRVArgDesc(SpvTy, Acc);
+    }
+    llvm_unreachable("Unexpected spirv target extension type");
+  }
+  llvm_unreachable("Unexpected target extension type");
+}
+#endif //VC_INTR_LLVM_VERSION_MAJOR >= 16
+
 static SPIRVArgDesc analyzeKernelArg(const Argument &Arg) {
   const Function *F = Arg.getParent();
   // If there is vc attribute, then no conversion is needed.
@@ -264,6 +310,10 @@ static SPIRVArgDesc analyzeKernelArg(const Argument &Arg) {
     return {SPIRVType::None};
 
   Type *Ty = Arg.getType();
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+  if (auto *TET = dyn_cast<TargetExtType>(Ty))
+    return analyzeTargetExtTypeArg(Arg, TET);
+#endif //VC_INTR_LLVM_VERSION_MAJOR >= 16
   // Not a pointer means that it is general argument without annotation.
   if (!isa<PointerType>(Ty))
     return {SPIRVType::Other};
@@ -275,6 +325,9 @@ static SPIRVArgDesc analyzeKernelArg(const Argument &Arg) {
       AddressSpace != SPIRVParams::SPIRVConstantAS)
     return {SPIRVType::Other};
 
+  if (VCINTR::Type::isOpaquePointerTy(Ty))
+    return {SPIRVType::Pointer};
+
   Type *PointeeTy = VCINTR::Type::getNonOpaquePtrEltTy(PointerTy);
   // Not a pointer to struct, cannot be sampler or image.
   if (!isa<StructType>(PointeeTy))
@@ -406,6 +459,8 @@ static std::string mapSPIRVDescToArgDesc(SPIRVArgDesc SPIRVDesc) {
 static PointerType *getKernelArgPointerType(PointerType *ConvertTy,
                                             PointerType *ArgTy) {
   auto AddressSpace = ConvertTy->getPointerAddressSpace();
+  if (VCINTR::Type::isOpaquePointerTy(ArgTy))
+    return VCINTR::PointerType::getWithSamePointeeType(ArgTy, AddressSpace);
   auto *ConvertPointeeTy = VCINTR::Type::getNonOpaquePtrEltTy(ConvertTy);
   auto *ArgPointeeTy = VCINTR::Type::getNonOpaquePtrEltTy(ArgTy);
 
@@ -493,10 +548,13 @@ static void rewriteKernelArguments(Function &F) {
   // 2. Kernel is referenced in @llvm.global.annotations
   //    We have to replace the original function with the new one
   if (!F.use_empty()) {
-    assert(F.hasOneUse());
-    auto *Bitcast = F.user_back();
-    assert(Bitcast->hasOneUse());
-    auto *Struct = Bitcast->user_back();
+    Value *Ptr = &F;
+    assert(Ptr->hasOneUse());
+    if (isa<BitCastOperator>(Ptr->user_back())) {
+      Ptr = Ptr->user_back();
+      assert(Ptr->hasOneUse());
+    }
+    auto *Struct = Ptr->user_back();
     assert(Struct->hasOneUse());
     auto *Array = Struct->user_back();
     assert(Array->hasOneUse());
